import LibHanger.Library.uwLogger as Logger
import LibHanger.Library.uwUtil as uwUtil
from LibHanger.Library.uwMail import uwMail
from LibHanger.Models.recset import recset
from netkeiber.Models.trn_race_id import trn_race_id
from netkeiber.Models.trn_race_info import trn_race_info

class netkeiberMailer():
    
    """
    netkeiber - メールクラス
    """
    
    class nkMessage():
        
        """
        メッセージクラス
        """
        
        def __init__(self):
            
            """
            コンストラクタ
            """
            
            self.mail_from = ''
            """ 送信元メールアドレス """

            self.mail_to = ''
            """ 送信先メールアドレス """
            
            self.subject = []
            """ 件名 """
            
            self.bodyText = []
            """ 本文 """
    
    class nkMessageTemplate():
        
        """
        メール定型文クラス
        """
        
        class mailMessage():
            
            """
            メール定型文
            """
            
            subjectTemplate = '[netkeiber] proc:{0} result:{1}'
            """ 件名テンプレート 
            {0}:処理内容
            {1}:処理結果(success/warning/error)
            """
            
            diskFreeInfo = 'diskfree: {0}%'
            """ 空きディスク容量 
            {0}:空きディスク容量(%)
            """
            
            importDataCal = 'year:{0} \n race_id_count:{1} \n scraping_count:{2}'
            """ 取込データカレンダー情報 
            {0}:取得年
            {1}:取得race_id件数
            {2}:スクレイピング回数
            """
            
            importDataInfoHeaderH = 'race_id,racecourse_id,race_no,race_nm,scraping_count,get_time,result'
            importDataInfoHeaderL = '--------------------------------------------------------------------'
            importDataInfoDetailD = '{0[0]};{0[1]};{0[2]}R;{0[3]};{0[4]};{0[5]};{0[6]}'
            """ 取込データ情報 
            {0}:レースID
            {1}:競馬場ID
            {2}:レース番号
            {3}:レース名
            {4}:スクレイピング回数
            {5}:取得に要した時間
            {6}:処理結果
            """
            
    def __init__(self, __um:uwMail):
        
        """
        コンストラクタ
        """

        # uwMailクラスインスタンス取得
        self.um = __um
    
    def sendMail(self, msg:nkMessage):
        
        """
        メール送信
        """
        
        try:
            # メール送信
            subject = '\n'.join(msg.subject)
            bodyText = '\n'.join(msg.bodyText)
            self.um.sendMail(msg.mail_from, msg.mail_to, subject, bodyText)
        except Exception as e:
            print(str(e))
            Logger.logging.error(e)

    def getSubject(self, proc, result)->list:
        
        msgLine = []
        msgLine.append(self.nkMessageTemplate.mailMessage.subjectTemplate.format(proc,result))
        return msgLine
    
    def addMessageDiskfree(self, msgLine:list):

        # 空きディスク容量取得
        diskFree = uwUtil.dskFree

        msgLine.append('====================================================')
        msgLine.append(self.nkMessageTemplate.mailMessage.diskFreeInfo.format(str(diskFree)))
        msgLine.append('====================================================')
    
    def getBodyTextByRaceIdGetter(self, year, race_id_count, scrapingCount)->list:
        
        msgLine = []
        # 取得メッセージ
        msgLine.append(self.nkMessageTemplate.mailMessage.importDataCal.format(str(year),str(race_id_count),str(scrapingCount)))
        # ディスク空き容量
        self.addMessageDiskfree(msgLine)
        return msgLine
    
    def getBodyTextByRaceInfoGetter(self, msgLine)->list:

        # ディスク空き容量
        self.addMessageDiskfree(msgLine)
        
        return msgLine
    


from Scrapinger.Library.scrapingConfig import scrapingConfig

class netkeiberConfig(scrapingConfig):
    
    """
    netkeiber共通設定クラス(netkeiberConfig)
    """ 
    
    class settingValueStruct(scrapingConfig.settingValueStruct):

        """
        設定値構造体
        """ 

        class SearchKeyword:
            
            """
            SearchKeyword
            """ 
            
            race = 'race/'
            """ レース検索 """

            horse = 'horse/'
            """ 競走馬検索 """

            jockey = 'jockey/'
            """ 騎手検索 """

            trainer = 'trainer/'
            """ 調教師検索 """
            
            howner = 'howner/'
            """ 馬主検索 """
            
            open = 'race/sum/'
            """ 開催情報検索 """

            refund = 'race/pay/'
            """ 払戻情報検索 """
            
            race_id_cal = 'top/calendar.html?year={0}&month={1}'
            """ レースID情報検索(カレンダー) """

            race_id_kaisai_date = 'top/race_list.html?kaisai_date={0}'
            """ レースID情報検索(開催日) """
            
            def __init__(self):
                
                """ 
                コンストラクタ
                """ 

                # メンバ変数初期化
                self.race = ''
                self.horse = ''
                self.jockey = ''
                self.trainer = ''
                self.howner = ''
                self.open = ''
                self.refund = ''
                self.race_id_cal = ''
                self.race_id_kaisai_date = ''

        class MailConfig(scrapingConfig.settingValueStruct.MailConfig):
            
            """
            MailConfig
            """
            
            mail_from = ''
            mail_to = ''
            
            def __init__(self):
                
                """
                コンストラクタ
                """
                
                super().__init__()
                
                self.mail_from = ''
                """ 送信元メールアドレス """
            
                self.mail_to = ''
                """ 送信先メールアドレス """

        class ArrivalOrder:
            
            """
            ArrivalOrder(着順)
            """ 

            raceCancel = 96
            """ レース発走中止 """
            raceCancelJp1 = ''
            raceCancelJp2 = '取'
            """ レース発走中止(日本語) """
            
            exclusion = 97
            """ 除外 """
            exclusionJp = '除'
            """ 除外(日本語) """

            disqualification = 98
            """ 失格 """
            disqualificationJp = '失'
            """ 失格(日本語) """

            stop_competition = 99
            """ 競争中止 """
            stop_competitionJp = '中'
            """ 競争中止(日本語) """
            
        class InCommingFlg:
            
            """
            InCommingFlg(入線フラグ)
            """ 
            
            defaultInLine = 0
            """ 通常入線 """

            accretion = 1
            """ 降着 """
            accretionJp = '降'
            """ 降着(日本語名) """

            exclusion = 2
            """ 除外 """

            disqualification = 3
            """ 失格 """

            stop_competition = 4
            """ 競争中止 """

            cancel_race = 5
            """ レース発走中止 """

        class Grade:
            
            """
            Grade(グレード)
            """ 

            defaultGrade = 0
            """ 平場 """

            g1Grade = 1
            """ G1 """

            g2Grade = 2
            """ G1 """

            g3Grade = 3
            """ G3 """
        
        class Direction:
        
            Steeplechase = '障'
            """ 障害 """
            
            left = '左'
            """ 左回り """
            
            right = '右'
            """ 右回り """
        
        class GroundKbn:
            
            Turf = 0
            """ 芝 """
            
            Dirt = 1
            """ ダート """

            Steeplechase_Turf = 2
            """ 障害(芝) """
        
            Steeplechase_TurfAndDirt = 3
            """ 障害(芝+ダート) """

            Steeplechase_Dirt = 4
            """ 障害(ダート) """

            TurfAbbr = '芝'
            """ 芝(略称) """
            
            DirtAbbr = 'ダ'
            """ ダート(略称) """
            
            SteeplechaseAbbr = '障'
            """ 障害(略称) """
        
        class PayInfoClass:
            
            win = 'tan'
            """ 単勝 """
            
            dwin = 'fuku'
            """ 複勝 """

            fcwin = 'waku'
            """ 枠連 """

            hcwin = 'uren'
            """ 馬連 """

            wdwin = 'wide'
            """ ワイド """

            hswin = 'utan'
            """ 馬単 """

            tdwin = 'sanfuku'
            """ 三連複 """

            tswin = 'santan'
            """ 三連単 """
            
    def __init__(self):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()

        self.netkeibaUrl = 'https://db.netkeiba.com/'
        """ netkeibaURL """

        self.netkeibaUrl_race = 'https://race.netkeiba.com/'
        """ netkeibaURL(race) """
        
        self.netkeibaUrlSearchKeyword = self.settingValueStruct.SearchKeyword()
        """ netkeiba URL Search Keyword """

        self.netkeibaUrlSearchKeyword.race = 'race/'
        """ netkeiba URL Search Keyword - race """

        self.netkeibaUrlSearchKeyword.horse = 'horse/'
        """ netkeiba URL Search Keyword - horse """

        self.netkeibaUrlSearchKeyword.jockey = 'jockey/'
        """ netkeiba URL Search Keyword - jockey """

        self.netkeibaUrlSearchKeyword.trainer = 'trainer/'
        """ netkeiba URL Search Keyword - trainer """

        self.netkeibaUrlSearchKeyword.howner = 'howner/'
        """ netkeiba URL Search Keyword - howner """

        self.netkeibaUrlSearchKeyword.open = 'race/sum/'
        """ netkeiba URL Search Keyword - open """

        self.netkeibaUrlSearchKeyword.refund = 'race/pay/'
        """ netkeiba URL Search Keyword - pay """
        
        self.netkeibaMailConfig = self.settingValueStruct.MailConfig()
        """ netkeiba Mail Config """

        self.netkeibaMailConfig.mail_from = ''
        """ netkeiba Mail Config - mail_from """

        self.netkeibaMailConfig.mail_to = ''
        """ netkeiba Mail Config - mail_to """
        
        self.LimitsScrapingCount = 0
        """ Limits Scraping Count """
        
        self.courseList:dict = {'札幌': '01',
                                '函館': '02',
                                '福島': '03',
                                '新潟': '04',
                                '東京': '05',
                                '中山': '06',
                                '中京': '07',
                                '京都': '08',
                                '阪神': '09',
                                '小倉': '10'}
        """ 競馬場リスト """
        
    def getConfigFileName(self):
        
        """ 
        設定ファイル名 
        """

        return 'netkeiber.ini'
    
    def setInstanceMemberValues(self):
        
        """ 
        インスタンス変数に読み取った設定値をセットする
        """
        
        # 基底側実行
        super().setInstanceMemberValues()
        
        # netkeibaURL
        super().setConfigValue('netkeibaUrl',self.config_ini,'SITE','NETKEIBA_URL',str)

        # netkeibaURL(race)
        super().setConfigValue('netkeibaUrl_race',self.config_ini,'SITE','NETKEIBA_URL_RACE',str)

        # netkeibaURL Search Keyword - race
        super().setConfigValue('netkeibaUrlSearchKeyword.race',self.config_ini,'SEARCH_KEYWORD','RACE',str)

        # netkeibaURL Search Keyword - horse
        super().setConfigValue('netkeibaUrlSearchKeyword.horse',self.config_ini,'SEARCH_KEYWORD','HORSE',str)

        # netkeibaURL Search Keyword - jockey
        super().setConfigValue('netkeibaUrlSearchKeyword.jockey',self.config_ini,'SEARCH_KEYWORD','JOCKEY',str)

        # netkeibaURL Search Keyword - trainer
        super().setConfigValue('netkeibaUrlSearchKeyword.trainer',self.config_ini,'SEARCH_KEYWORD','TRAINER',str)

        # netkeibaURL Search Keyword - howner
        super().setConfigValue('netkeibaUrlSearchKeyword.howner',self.config_ini,'SEARCH_KEYWORD','HOWNER',str)

        # netkeibaURL Search Keyword - open
        super().setConfigValue('netkeibaUrlSearchKeyword.open',self.config_ini,'SEARCH_KEYWORD','OPEN',str)

        # netkeibaURL Search Keyword - refund
        super().setConfigValue('netkeibaUrlSearchKeyword.refund',self.config_ini,'SEARCH_KEYWORD','REFUND',str)

        # netkeibaURL Search Keyword - race_id_cal
        super().setConfigValue('netkeibaUrlSearchKeyword.race_id_cal',self.config_ini,'SEARCH_KEYWORD','RACE_ID_CAL',str)

        # netkeibaURL Search Keyword - race_id_kaisai_date
        super().setConfigValue('netkeibaUrlSearchKeyword.race_id_kaisai_date',self.config_ini,'SEARCH_KEYWORD','RACE_ID_KAISAI_DATE',str)

        # netkeiba MailConfig - mail_from
        super().setConfigValue('netkeibaMailConfig.mail_from',self.config_ini,'MAIL_CONFIG','MAIL_FROM',str)

        # netkeiba MailConfig - mail_to
        super().setConfigValue('netkeibaMailConfig.mail_to',self.config_ini,'MAIL_CONFIG','MAIL_TO',str)

        # netkeiba Limits Scraping Count
        super().setConfigValue('LimitsScrapingCount',self.config_ini,'SITE','LIMITS_SCRAPING_COUNT_ONEDAY',int)



from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Models.recset import recset
from LibHanger.Models.saWhere import saWhere
from netkeiber.Getter.Base.baseGetter import baseGetter
from netkeiber.Getter.trn_race_result import getter_trn_race_result
from netkeiber.Getter.trn_horse_result import getter_trn_horse_result
from netkeiber.Getter.trn_open_info import getter_trn_open_info
from netkeiber.Getter.trn_refund_info import getter_trn_refund_info
from netkeiber.Getter.log_race_id import getter_log_race_id
from netkeiber.Models.trn_horse_result import trn_horse_result
from netkeiber.Models.trn_race_info import trn_race_info
from netkeiber.Models.trn_open_info import trn_open_info
from netkeiber.Models.trn_refund_info import trn_refund_info
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Library.netkeiberException import getterError

class getter_RaceData(baseGetter):
    
    """
    レースデータ取得クラス
    """
    
    def __init__(self, __psgr:uwPostgreSQL):
        
        """
        コンストラクタ
        
        Parameters
        ----------
        __psgr : uwPostgreSQL
            uwPostgreSQLクラスインスタンス
        """
        
        # uwPostgreSQL
        self.__psgr = __psgr

        # レコードセット初期化
        self.init_recset()
        
        # 取得フラグ
        self.horseResultRead = True
        self.openInfoRead = True
        self.refundInfoRead = True
        
        # エラー有無
        self.__hasError = False

        # スクレイピング回数
        self.__scrapingCount = 0
    
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レース結果取得クラスインスタンス
        self.raceResult = getter_trn_race_result()
        self.raceResult.rsRaceResult.setDA(self.__psgr)
        
        # 競走馬成績取得クラスインスタンス
        self.horseResult = getter_trn_horse_result()
        self.horseResult.rsHorseResult.setDA(self.__psgr)

        # 開催情報取得クラスインスタンス
        self.openInfo = getter_trn_open_info()
        self.openInfo.rsOpenInfo.setDA(self.__psgr)
        
        # 払戻情報取得クラスインスタンス
        self.refundInfo = getter_trn_refund_info()
        self.refundInfo.rsRefundInfo.setDA(self.__psgr)
        
        # レースIDログ
        self.raceIdLog = getter_log_race_id()

    @property
    def scrapingCount(self):
        
        """
        スクレイピング回数
        """
        
        return self.__scrapingCount
    
    @property
    def hasError(self):
        
        """
        エラー有無
        """
        
        return self.__hasError
    
    def getData(self, race_id, racecourse_id):
        
        """
        データ取得
        
        Parameters
        ----------
        race_id : str
            レースID
        racecourse_id : str
            競馬場ID
        """
        
        try:
            
            # エラーフラグ初期化
            self.__hasError = False
            
            # session - open
            self.__psgr.openSession()

            # レース結果取得
            self.raceResult.getData(race_id=race_id)
            
            # レース結果の取得件数が0件だった場合は処理を抜ける
            if self.raceResult.rsRaceResult.recordCount == 0:
                return

            # saWhereインスタンス
            saw = saWhere()
            
            # レース結果ループ開始
            rsRaceResult = self.raceResult.rsRaceResult
            rsRaceInfo = self.raceResult.rsRaceInfo
            rsRaceInfo.first()
            while rsRaceResult.eof() == False:
                
                # 競走馬成績取得
                # 取得済データチェック
                saw.clear()
                saw.and_(trn_horse_result.horse_id == rsRaceResult.fields(trn_horse_result.horse_id.key).value) \
                   .and_(trn_horse_result.race_id == rsRaceResult.fields(trn_horse_result.race_id.key).value)
                if self.horseResultRead == True and \
                   self.horseResult.rsHorseResult.find(saw, recset.findOption.cacheAndDataBase) == False:
                    
                    # 競走馬成績取得
                    self.horseResult.getData(horse_id=rsRaceResult.fields(trn_horse_result.horse_id.key).value)
                    
                # 開催情報取得
                if self.openInfoRead:
                    dfOpenIdGroup = self.horseResult.rsHorseResult.dfrows.groupby(trn_open_info.open_id.key)
                    for open_id, group in dfOpenIdGroup:
                        
                        # 取得済データチェック
                        rsOpenInfo = self.openInfo.rsOpenInfo
                        saw.clear()
                        saw.and_(trn_open_info.open_id == open_id) \
                        .and_(trn_open_info.racecourse_id == racecourse_id)
                        if rsOpenInfo.find(saw, recset.findOption.cacheAndDataBase) == False:
                            
                            # 開催情報取得
                            self.openInfo.getData(open_id=open_id, racecourse_id=racecourse_id)
                
                # 払戻情報取得
                if self.refundInfoRead:
                    
                    # 払戻ID取得
                    refund_id = rsRaceInfo.fields(trn_race_info.refund_id.key).value

                    # 取得済データチェック
                    rsRefundInfo = self.refundInfo.rsRefundInfo
                    saw.clear()
                    saw.and_(trn_refund_info.refund_id == refund_id) \
                    .and_(trn_refund_info.racecourse_id == racecourse_id) \
                    .and_(trn_refund_info.race_no == rsRaceInfo.fields(trn_race_info.race_no.key).value)
                    if rsRefundInfo.find(saw, recset.findOption.cacheAndDataBase) == False:
                        
                        # 払戻情報取得
                        self.refundInfo.getData(refund_id=refund_id, racecourse_id=racecourse_id)

            # スクレイピング回数取得
            rsList = []
            rsList.append(self.raceResult.scrapingCount)
            rsList.append(self.horseResult.scrapingCount)
            rsList.append(self.openInfo.scrapingCount)
            rsList.append(self.refundInfo.scrapingCount)
            self.__scrapingCount = sum(rsList)

        except getterError as geException:
            self.__hasError = True
            self.raceIdLog.setLogRaceId(race_id, nd.getterResult.error ,geException)
        except Exception as e:
            self.__hasError = True
            self.raceIdLog.setLogRaceId(race_id, nd.getterResult.error ,e)
        finally:
            # session - close
            self.__psgr.closeSession()



import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberException import getterError
from netkeiber.Models.trn_open_info import trn_open_info
from netkeiber.Getter.Base.baseGetter import baseGetter

class getter_trn_open_info(baseGetter):
    
    """
    開催情報取得クラス
    (trn_open_info)
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()

        # レコードセット初期化
        self.init_recset()
    
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsOpenInfo = recset[trn_open_info](trn_open_info)

    @Logger.loggerDecorator("getData",['open_id'])
    def getData(self, *args, **kwargs):
        
        """
        開催情報取得
        
        Parameters
        ----------
        kwargs : dict
            @open_id
                開催ID
        """
        
        # 開催情報をDataFrameで取得
        try:
            kwargs['getter'] = self
            result = self.getOpenInfoDataToDataFrame(**kwargs)
        except Exception as e:
            raise getterError(e)
        return result
    
    @Logger.loggerDecorator("getOpenInfoDataToDataFrame")
    def getOpenInfoDataToDataFrame(self, *args, **kwargs):

        """
        開催情報取得
        
        Parameters
        ----------
        kwargs : dict
            @open_id
                開催ID
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl, gv.netkeiberConfig.netkeibaUrlSearchKeyword.open)
        # 検索url(開催情報)
        openInfoUrl = urlParse.urljoin(rootUrl, kwargs.get('racecourse_id') + '/' + kwargs.get('open_id'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(openInfoUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)
    
    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """

        def __init__(self, _config: netkeiberConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : netkeiberConfig
                    共通設定
            """

            super().__init__(_config)
            
            self.config = _config
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup
            
        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            開催情報をDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @open_id
                    取得対象開催ID
            """

            # 開催ID取得
            open_id:str = kwargs.get('open_id')

            # getterインスタンス取得
            self.bc:getter_trn_open_info = kwargs.get('getter')
            
            # 競馬場ID取得
            racecourse_id:str = kwargs.get('racecourse_id')
            
            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            tables = soup.find(class_="race_table_01").find_all('tr')

            if tables:
                
                # 開催情報model用意
                openInfo = recset[trn_open_info](trn_open_info)
                
                for index in range(len(tables)):
                    if index == 0 : continue
                    try:
                        row = tables[index].find_all('td')
                        # レースNO
                        race_no = row[0].text
                        # レースID
                        race_id = str(row[1].find_all('a')[0].get('href')).split('/')[2]
                        # 勝ち馬ID
                        win_horse = str(row[3].find_all('a')[0].get('href')).split('/')[2]
                        # 勝ち馬騎手ID
                        win_jockey = str(row[3].find_all('a')[1].get('href')).split('/')[2]
                        # 2着馬ID
                        snd_horse = str(row[4].find_all('a')[0].get('href')).split('/')[2]
                        # 2着馬騎手ID
                        snd_jockey = str(row[4].find_all('a')[1].get('href')).split('/')[2]
                        # 3着馬ID
                        trd_horse = str(row[5].find_all('a')[0].get('href')).split('/')[2]
                        # 3着馬騎手ID
                        trd_jockey = str(row[5].find_all('a')[1].get('href')).split('/')[2]

                        # Modelに追加
                        openInfo.newRow()
                        openInfo.fields(trn_open_info.open_id.key).value = open_id
                        openInfo.fields(trn_open_info.racecourse_id.key).value = racecourse_id
                        openInfo.fields(trn_open_info.race_no.key).value = race_no
                        openInfo.fields(trn_open_info.race_id.key).value = race_id
                        openInfo.fields(trn_open_info.win_horse.key).value = win_horse
                        openInfo.fields(trn_open_info.win_jockey.key).value = win_jockey
                        openInfo.fields(trn_open_info.snd_horse.key).value = snd_horse
                        openInfo.fields(trn_open_info.snd_jockey.key).value = snd_jockey
                        openInfo.fields(trn_open_info.trd_horse.key).value = trd_horse
                        openInfo.fields(trn_open_info.trd_jockey.key).value = trd_jockey
                        openInfo.fields(trn_open_info.updinfo.key).value = self.bc.getUpdInfo()
                        
                        # コンソール出力
                        print('開催ID={0}'.format(open_id), end='レースNo={0}'.format(race_no))
                        
                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                        raise 

                # レコードセットマージ
                self.bc.rsOpenInfo.merge(openInfo)

                # 戻り値を返す
                return openInfo.getDataFrame()


from LibHanger.Library.uwGlobals import configer
from LibHanger.Library.uwGlobals import *
from netkeiber.Library.netkeiberGlobals import *

class netkeiberConfiger(configer):
    
    """
    netkeiber共通設定クラス
    """
    
    def __init__(self, tgv:netkeiberGlobal, file, configFolderName):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__(tgv, file, configFolderName)
        
        # netkeibar.ini
        da = netkeiberConfig()
        da.getConfig(file, configFolderName)

        # gvセット
        tgv.netkeiberConfig = da
        


import re
import LibHanger.Library.uwGetter as CmnGetter
from LibHanger.Models.fields import *
from Scrapinger.Library.browserContainer import browserContainer
from Scrapinger.Library.webDriverController import webDriverController
from netkeiber.Library.netkeiberGlobals import *

class netkeibaBrowserController(browserContainer):
    
    """
    netkeiberブラウザコントローラー
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()
                
        # WebDriverController
        self.wdc = webDriverController(gv.netkeiberConfig, self)
    
    def getData(self, *args, **kwargs):
        
        """
        データ取得
        """
        
        pass
    
    def getUpdInfo(self):

        """
        更新情報取得
        """

        return CmnGetter.getNow().strftime('%Y/%m/%d %H:%M:%S')
    
    def isdigitEx(self, targetString:str) -> bool:
        
        """
        数値判定
        """
        
        return re.compile("^\d+\.?\d*\Z").match(targetString)
    


import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from enum import Enum
from decimal import Decimal
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup, ResultSet, Tag
from sqlalchemy.sql.elements import Null
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberException import getterError
from netkeiber.Library.netkeiberException import gettingValueError
from netkeiber.Models.trn_horse_result import trn_horse_result
from netkeiber.Getter.Base.baseGetter import baseGetter

class getter_trn_horse_result(baseGetter):
    
    """
    競走馬成績取得クラス
    (trn_horse_result)
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__()

        # レコードセット初期化
        self.init_recset()
    
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsHorseResult = recset[trn_horse_result](trn_horse_result)
        
    @Logger.loggerDecorator("getData",['horse_id'])
    def getData(self, *args, **kwargs):
        
        """
        競争馬成績取得
        
        Parameters
        ----------
        kwargs : dict
            @horse_id
                取得対象競走馬ID
        """
        
        # 競争馬成績をDataFrameで取得
        try:
            kwargs['getter'] = self
            result = self.getHorseDataToDataFrame(**kwargs)
        except:
            raise getterError
        return result

    @Logger.loggerDecorator("getHorseDataToDataFrame")
    def getHorseDataToDataFrame(self, *args, **kwargs) -> baseGetter.getterResult:

        """
        競争馬成績取得
        
        Parameters
        ----------
        kwargs : dict
            @horse_id
                取得対象競走馬ID
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl, gv.netkeiberConfig.netkeibaUrlSearchKeyword.horse)
        # 検索url(競走馬成績)
        horseUrl = urlParse.urljoin(rootUrl, kwargs.get('horse_id'))

        # スクレイピング準備
        if self.wdc.browserCtl is None:
            self.wdc.settingScrape()
        
        # ページロード
        self.wdc.browserCtl.loadPage(horseUrl)
        
        # pandasデータを返却する
        self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

        # 処理結果追加
        return self.addResult(trn_horse_result.__tablename__, self.rsHorseResult.recordCount, self.rsHorseResult)
        
    def getPace(self, paceString:str):
        
        """
        ペース取得
        
        Parameters
        ----------
            paceString : str
                ペース文字列
        """
        
        paces = paceString.split('-') if paceString.replace('\xa0','') != '' else []
        return paces

    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """

        class horseResultCol(Enum):
            
            """
            競走馬成績列インデックス
            """
            
            racecourseId = 1
            """ 競馬場(開催)ID """
            
            raceId = 4
            """ レースID"""

            horseNo = 8
            """ 馬番 """
            
            paces = 21
            """ ペース """
            
            winHorseId = 26
            """ 勝馬ID """

        def __init__(self, _config: netkeiberConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : netkeiberConfig
                    共通設定
            """

            super().__init__(_config)
            
            self.config = _config
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup
            
        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            競走馬成績をDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @horse_id
                    取得対象競走馬ID
                @race_id
                    取得対象レースID
            """

            # 競走馬ID取得
            horse_id:str = kwargs.get('horse_id')
            
            # getterインスタンス取得
            self.bc:getter_trn_horse_result = kwargs.get('getter')

            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            horseResultTables = soup.find(class_="db_h_race_results").find_all('tr')
            if horseResultTables:
                
                # 競走馬成績model用意
                horseResult = recset[trn_horse_result](trn_horse_result)
                
                for index in range(len(horseResultTables)):
                    if index == 0 : continue
                    try:
                        dataTbl:Tag = horseResultTables[index]
                        row:ResultSet = dataTbl.find_all('td')
                        drow = self.getRow(row)
                        # レースID
                        race_id = str(drow[self.horseResultCol.raceId.value].find_all('a')[0].get('href')).split('/')[2]
                        # 競馬場ID
                        racecourse_id = str(drow[self.horseResultCol.racecourseId.value].find_all('a')[0].get('href')).split('/')[3]
                        # 開催ID
                        open_id = str(drow[self.horseResultCol.racecourseId.value].find_all('a')[0].get('href')).split('/')[4]
                        # 開催名称
                        open_nm = drow[self.horseResultCol.racecourseId.value].text
                        # 馬番
                        horse_no = Decimal(drow[self.horseResultCol.horseNo.value].text)
                        # ペース
                        paces = self.bc.getPace(drow[self.horseResultCol.paces.value].text)
                        # 勝ち馬ID
                        win_horse_ids = drow[self.horseResultCol.winHorseId.value].find_all('a')
                        win_horse_id = str(win_horse_ids[0].get('href')).split('/')[2] if len(win_horse_ids) > 0 else ''
                            
                        # Modelに追加
                        horseResult.newRow()
                        horseResult.fields(trn_horse_result.horse_id.key).value = horse_id
                        horseResult.fields(trn_horse_result.race_id.key).value = race_id
                        horseResult.fields(trn_horse_result.horse_no.key).value = horse_no
                        horseResult.fields(trn_horse_result.racecourse_id.key).value = racecourse_id
                        horseResult.fields(trn_horse_result.open_id.key).value = open_id
                        horseResult.fields(trn_horse_result.open_nm.key).value = open_nm
                        horseResult.fields(trn_horse_result.pace_firsth.key).value = paces[0] if len(paces) >= 1 else Null()
                        horseResult.fields(trn_horse_result.pace_latterh.key).value = paces[1] if len(paces) >= 2 else Null()
                        horseResult.fields(trn_horse_result.win_horse_id.key).value = win_horse_id
                        horseResult.fields(trn_horse_result.updinfo.key).value = self.bc.getUpdInfo()
                        
                        # コンソール出力
                        print('競走馬ID={0}'.format(horse_id))
                        print('レースID={0}'.format(race_id))
                        print('馬番={0}'.format(horse_no))
                        
                    except gettingValueError as gvException: # 値例外
                        Logger.logging.error(str(gvException))
                        raise 
                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                        raise 
                
                # レコードセットマージ
                self.bc.rsHorseResult.merge(horseResult)
                
                # 戻り値を返す
                return horseResult.getDataFrame()


from enum import Enum
from LibHanger.Library.uwDeclare import uwDeclare

class netkeiberDeclare(uwDeclare):
    
    """
    netkeiber - 定数クラス
    """
    
    class getterResult(Enum):
        
        """
        取得処理結果
        """
        
        success = 0
        """ 正常 """
        
        error = 1
        """ エラーあり """

        warning = 2
        """ 警告あり """
        
    class getStatus(Enum):
        
        """
        取得状況
        """
        
        unacquired = 0
        """ 未取得 """
        
        acquired = 1
        """ 取得済 """
        
    class refundKbn(Enum):
        
        """
        払戻区分
        """
        
        win = 0
        """ 単勝 """
        
        dwin = 1
        """ 複勝 """
        
        fcwin = 2
        """ 枠連 """
        
        hcwin = 3
        """ 馬連 """
        
        wdwin = 4
        """ ワイド """
        
        hswin = 5
        """ 馬単 """
        
        tdwin = 6
        """ 三連複 """
        
        tswin = 7
        """ 三連単 """
    


import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class trn_race_result(Base):
	
	# テーブル名
	__tablename__ = 'trn_race_result'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	race_id = fld.CharFields(12,primary_key=True,default='')
	horse_no = fld.NumericFields(2,0,primary_key=True,default=0)
	arrival_order = fld.NumericFields(2,0,default=0)
	incoming_flg = fld.NumericFields(1,0,default=0)
	frame_no = fld.NumericFields(2,0,default=0)
	horse_id = fld.CharFields(10,default=Null)
	horse_nm_en = fld.CharFields(30,default='')
	sex_age = fld.CharFields(10,default='')
	weight = fld.NumericFields(3,0,default=0)
	jockey_id = fld.CharFields(5,default='')
	time = fld.CharFields(10,default='')
	time_second = fld.NumericFields(3,0,default=0)
	arr_diff = fld.CharFields(10,default='')
	barr_diff_num = fld.NumericFields(3,1,default=0)
	tarr_diff_num = fld.NumericFields(3,1,default=0)
	corner1 = fld.NumericFields(1,0,default=Null)
	corner2 = fld.NumericFields(1,0,default=Null)
	corner3 = fld.NumericFields(1,0,default=Null)
	corner4 = fld.NumericFields(1,0,default=Null)
	last3furlong = fld.NumericFields(3,1,default=0)
	win_odds = fld.NumericFields(5,2,default=0)
	popular = fld.NumericFields(2,0,default=0)
	horse_weight = fld.NumericFields(3,0,default=0)
	weight_diff = fld.NumericFields(2,0,default=0)
	trainer_id = fld.CharFields(5,default='')
	howner_id = fld.CharFields(6,default='')
	prize_money = fld.NumericFields(6,1,default=0)
	updinfo = fld.CharFields(40,default='')



import os
import sys
import time
import LibHanger.Library.uwGetter as Getter
import LibHanger.Library.uwLogger as Logger
import LibHanger.Library.uwMath as uwMath
import netkeiber.Library.netkeiberConfiger as hc
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Library.uwDeclare import uwDeclare as en
from LibHanger.Library.uwMail import uwMail
from LibHanger.Library.uwExport import JsonExporter
from LibHanger.Library.uwImport import JsonImporter
from LibHanger.Models.recset import recset
from LibHanger.Models.saWhere import saWhere
from LibHanger.Models.saOrderBy import saOrderBy
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Models.trn_race_id import trn_race_id
from netkeiber.Models.trn_race_info import trn_race_info
from netkeiber.Getter.trn_race_id import getter_trn_race_id
from netkeiber.Getter.get_RaceData import getter_RaceData
from netkeiber.Register.register_RaceData import register_RaceData
from netkeiber.Library.netkeiberMailer import netkeiberMailer

def getAllRaceId(filePath, fromYear = 2021, toYear = 2007):
    
    """
    レースID取得メソッド(2008年～2021年)

    Parameters
    ----------
    filePath : str
        呼び出し元ファイルのパス
    fromYear : int
        取得年From
    toYear : int
        取得年To
    """

    # 共通設定
    hc.netkeiberConfiger(gv, filePath, 'config')
    
    # year.jsonファイル名定義
    yearJsonFileName = 'year.json'

    # year.jsonを格納するフォルダ名定義
    yearJsonDirName = 'json'
    
    # year.jsonの出力先ディレクトリチェック
    yearJsonDir = os.path.dirname(filePath)
    yearJsonDir = os.path.join(yearJsonDir, 'json')
    if not os.path.exists(yearJsonDir):
        os.mkdir(yearJsonDir)

    # 取込済年dict初期化
    yearDict = dict()
    
    # importer,exporterクラスインスタンス
    je = JsonExporter(filePath)
    ji = JsonImporter(filePath)

    # スクレイピング回数累計初期化
    scountCumu = 0

    # 2021->2008までループしてtrn_race_idを生成
    for year in range(fromYear, toYear, -1):
    
        # 取込済の年取得
        yearJsonFilePath = os.path.join(yearJsonDir, yearJsonFileName)
        if (os.path.exists(yearJsonFilePath)):
            yearDict = ji.convertToDict(yearJsonDirName, yearJsonFileName)
        
        # 対象年がyearDictに存在するか
        if str(year) in yearDict:
            continue
        
        # trn_race_id取得
        scount, getResult = getRaceId(filePath, year)
        scountCumu += scount
        
        # 成功したらyear.jsonに出力
        if getResult:
            try:
                yearDict[str(year)] = scount
                je.putJsonFile(yearJsonDirName, yearJsonFileName, yearDict)
            except Exception as e:
                Logger.logging.error(str(e))
                Logger.logging.error('Output of year.json failed. The Process is terminated.')
                sys.exit()
                
        # スクレイピング回数上限値チェック(超過時はループを抜ける)
        if scountCumu > gv.netkeiberConfig.LimitsScrapingCount:
            Logger.logging.warning('Scraping count exceeded.')
            break
        
def getRaceId(filePath, year):
    
    """
    レースID取得メソッド(年単位)

    Parameters
    ----------
    filePath : str
        呼び出し元ファイルのパス
    year : int
        取得年
    """

    # 共通設定
    hc.netkeiberConfiger(gv, filePath, 'config')

    # レースID情報取得クラスインスタンス
    getTrnRaceId = getter_trn_race_id()

    # uwPostgreSQL
    psgr_trid = uwPostgreSQL(gv.config) # trn_race_id用

    # 開始ログ
    Logger.logging.info('>> Started the process of trn_race_id. year={0}'.format(str(year)))

    # レースID情報取得
    scountCumu = 0
    for month in range(1,13,1):
        Logger.logging.info('>>>> Started getTrnRaceId.getData. Month={0}'.format(str(month)))
        getTrnRaceId.getData(year=year, month=month)
        scountCumu = getTrnRaceId.scrapingCount
        Logger.logging.info('<<<< Finished getTrnRaceId.getData. Month={0} ScrapingCount={1}'.format(str(month),str(getTrnRaceId.scrapingCount)))
        if scountCumu > gv.netkeiberConfig.LimitsScrapingCount:
            return scountCumu, False

    # レコードセット退避
    rsRaceId = getTrnRaceId.rsRaceId
    
    # reg
    regRaceData = register_RaceData(psgr_trid)
    regRaceData.appendRecsetList(rsRaceId)

    # 処理結果初期化
    procResult = nd.getterResult.success

    # update
    Logger.logging.info('>>>> Started trn_race_id regist database')
    result = regRaceData.execUpdate()
    if result:
        Logger.logging.info('<<<< ◎trn_race_id Regist Success Count={0}'.format(str(rsRaceId.recordCount)))
    else:
        Logger.logging.info('<<<< ☓trn_race_id Regist Failed')
        procResult = nd.getterResult.error

    # 終了ログ
    Logger.logging.info('<< trn_race_id acquisition process has been completed. year={0} result={1}'.format(str(year), procResult))

    # メールインスタンス
    um = uwMail(gv.config.MailConfig.Host, 
                gv.config.MailConfig.SmtpPort, 
                gv.config.MailConfig.User, 
                gv.config.MailConfig.Password)

    # netkeiber - mail
    nkm = netkeiberMailer(um)

    # netkeiber - mailMsg
    mmsg = nkm.nkMessage()

    # mailMessage
    mmsg.mail_from = gv.netkeiberConfig.netkeibaMailConfig.mail_from
    mmsg.mail_to = gv.netkeiberConfig.netkeibaMailConfig.mail_to
    mmsg.subject = nkm.getSubject('trn_race_id acquisition process has been completed. year=[{0}]'.format(str(year)),procResult.name)
    mmsg.bodyText = nkm.getBodyTextByRaceIdGetter(year, rsRaceId.recordCount, scountCumu)
    # mailsend
    nkm.sendMail(mmsg)
    
    # スクレイピング回数累計を返す
    return scountCumu, True

def getRaceData(filePath, year, race_id = '*', open_id = '*'):
    
    """
    レースデータ取得メソッド(年単位)

    Parameters
    ----------
    filePath : str
        呼び出し元ファイルのパス
    year : int
        取得年
    race_id : str
        レースID
    open_id : str
        開催ID
    """

    # 共通設定
    hc.netkeiberConfiger(gv, filePath, 'config')
    
    # メールインスタンス
    um = uwMail(gv.config.MailConfig.Host, 
                gv.config.MailConfig.SmtpPort, 
                gv.config.MailConfig.User, 
                gv.config.MailConfig.Password)

    # uwPostgreSQL
    psgr_trid = uwPostgreSQL(gv.config) # trn_race_id用
    psgr_grad = uwPostgreSQL(gv.config) # getter用
    psgr_regt = uwPostgreSQL(gv.config) # register用

    # netkeiber - mail
    nkm = netkeiberMailer(um)

    # netkeiber - mailMsg
    mmsg = nkm.nkMessage()
    
    # 開始ログ
    Logger.logging.info('>> Started get trn_race_id. year={0}'.format(str(year)))

    # where
    ts = saWhere()
    if year >= 2008 and race_id == '*' and open_id == '*':
        w = ts.and_(trn_race_id.race_id >= str(year) + '00000000').and_(trn_race_id.race_id <= str(year) + '99999999')
        w = w.and_(trn_race_id.get_status == 0)
    elif year < 2007 and race_id == '*' and open_id == '*':
        print('year is invalid.')
        sys.exit()
    elif race_id != '*':
        w = ts.and_(trn_race_id.race_id == race_id)
        w = w.and_(trn_race_id.get_status == 0)
    elif open_id != '*':
        w = ts.and_(trn_race_id.open_id == open_id)
        w = w.and_(trn_race_id.get_status == 0)
    else:
        print('args is invalid.')
        sys.exit()

    # order by
    ob = saOrderBy()
    s = ob.asc_(trn_race_id.race_id)
    
    # 更新用レコードセット用意
    psgr_trid.openSession(True)
    rsRaceId = recset[trn_race_id](trn_race_id)
    rsRaceId.setDA(psgr_trid)
    rsRaceId.filter(w, s)

    # 終了ログ
    Logger.logging.info('<< Finished get trn_race_id. year={0}'.format(str(year)))
        
    # レースデータ取得クラスインスタンス
    getRaceData = getter_RaceData(psgr_grad)

    # 開始ログ
    Logger.logging.info('>> Started the process of acquiring race data. year={0}'.format(str(year)))

    # trn_race_idループ
    msgLine = []
    while rsRaceId.eof() == False:
        
        # 処理時間計測 - 開始
        start = time.perf_counter()
        
        # レースデータ取得:パラメーター設定
        getRaceData.horseResultRead = False # 競走馬成績は取得しない
        getRaceData.openInfoRead = False # 開催情報は取得しない
        race_id = rsRaceId.fields(trn_race_id.race_id.key).value
        racecourse_id = rsRaceId.fields(trn_race_id.racecourse_id.key).value
        open_id = rsRaceId.fields(trn_race_id.open_id.key).value

        # 処理結果初期化
        procResult = nd.getterResult.success
        
        # 開始ログ
        Logger.logging.info('>>>> □===== Started get race_id={0} ====='.format(race_id))

        # レースデータ取得
        getRaceData.init_recset()
        getRaceData.getData(race_id=race_id, racecourse_id=racecourse_id)

        # レコードセット退避
        rsRaceResult = getRaceData.raceResult.rsRaceResult
        rsRaceInfo = getRaceData.raceResult.rsRaceInfo
        rsMstHorse = getRaceData.raceResult.rsMstHourse
        rsMstJockey = getRaceData.raceResult.rsMstJockey
        rsMstTrainer = getRaceData.raceResult.rsMstTrainer
        rsMstHowner = getRaceData.raceResult.rsMstHowner
        rsHorseResult = getRaceData.horseResult.rsHorseResult
        rsOpenInfo = getRaceData.openInfo.rsOpenInfo
        rsRefundInfo = getRaceData.refundInfo.rsRefundInfo
        rsRaceIdLog = getRaceData.raceIdLog.rsLogRaceId

        # レースNo,レース名退避
        race_no = ''
        race_nm = ''
        if getRaceData.hasError == False:
            race_no = rsRaceInfo.fields(trn_race_info.race_no.key).value
            race_nm = rsRaceInfo.fields(trn_race_info.race_nm.key).value

        # rsRaceIdLog - setDA
        rsRaceIdLog.setDA(psgr_regt)

        # エラー有無判定
        if getRaceData.hasError == True:

            # 終了ログ
            Logger.logging.info('<<< ■===== Finished get race_id={0} (getter_RaceData is terminated with an error) ====='.format(race_id))
            
            # レースIDログ登録
            upResult = rsRaceIdLog.upsert()
            if upResult.result == en.resultRegister.success:
                Logger.logging.warning('<<<< ◎log_race_id registration process Succeeded. [Error]')
            elif upResult.result == en.resultRegister.failure:
                Logger.logging.error('<<<< ☓log_race_id registration process failed.')
            procResult = nd.getterResult.error
        else:
            
            # DB更新対象セット
            regRaceData = register_RaceData(psgr_regt, race_id)
            regRaceData.appendRecsetList(rsRaceResult)
            regRaceData.appendRecsetList(rsRaceInfo)
            regRaceData.appendRecsetList(rsMstHorse)
            regRaceData.appendRecsetList(rsMstJockey)
            regRaceData.appendRecsetList(rsMstTrainer)
            regRaceData.appendRecsetList(rsMstHowner)
            regRaceData.appendRecsetList(rsHorseResult)
            regRaceData.appendRecsetList(rsOpenInfo)
            regRaceData.appendRecsetList(rsRefundInfo)

            # 開始ログ
            Logger.logging.info('>>>> □===== Started race data registration process race_id={0} ====='.format(race_id))

            # update
            result = regRaceData.execUpdate()

            # 終了ログ
            Logger.logging.info('<<<< □===== Finished race data registration process race_id={0} ====='.format(race_id))

            if result == en.resultRegister.success:
                Logger.logging.info('<<<< ◎Race data registration process Succeeded.')
            elif result == en.resultRegister.failure:
                Logger.logging.error('<<<< ☓Race data registration process Failed.')
                procResult = nd.getterResult.error

            # 処理時間 - 取得
            procTime = time.perf_counter() - start

            # 開始ログ
            Logger.logging.info('>>>> □===== Started trn_race_id registration process race_id={0} ====='.format(race_id))

            # スクレイピング回数、データ取得時間セット
            rsRaceId.editRow()
            rsRaceId.fields(trn_race_id.scraping_count.key).value = getRaceData.scrapingCount
            rsRaceId.fields(trn_race_id.get_time.key).value = uwMath.round(procTime, uwMath.fraction.round)
            rsRaceId.fields(trn_race_id.get_status.key).value = nd.getStatus.acquired.value
            rsRaceId.fields(trn_race_id.updinfo.key).value = Getter.getNow(Getter.datetimeFormat.updinfo)

            # レースID情報登録
            riResult = rsRaceId.upsert()

            # 終了ログ
            Logger.logging.info('<<<< □===== Finished trn_race_id registration process race_id={0} ====='.format(race_id))

            if riResult.result == en.resultRegister.success:
                Logger.logging.info('<<<< ◎trn_race_id registration process Succeeded.')
            elif riResult.result == en.resultRegister.failure:
                Logger.logging.error('<<<< ☓trn_race_id registration process Failed.')
                procResult = nd.getterResult.warning

            # 開始ログ
            Logger.logging.info('>>>> □===== Started log_race_id registration process race_id={0} ====='.format(race_id))
            
            # レースIDログ登録
            rsRaceIdLog = regRaceData.raceIdLog.rsLogRaceId
            rsRaceIdLog.setDA(psgr_regt)
            upResult = rsRaceIdLog.upsert()
            
            # 終了ログ
            Logger.logging.info('<<<< □===== Finished log_race_id registration process race_id={0} ====='.format(race_id))
            
            if upResult.result == en.resultRegister.success:
                Logger.logging.info('◎log_race_id registration process Succeeded.')
            elif upResult.result == en.resultRegister.failure:
                Logger.logging.error('☓log_race_id registration process Failed.')
                procResult = nd.getterResult.warning

            # 終了ログ
            if procResult == nd.getterResult.success:
                Logger.logging.info('>>> □===== Finished get race_id={0} ====='.format(race_id))
            else:
                Logger.logging.warning('>>> ■===== Finished get race_id={0} (An error has occured)====='.format(race_id))
        
        # msgLine追加
        AddMailMessage(nkm, msgLine, rsRaceId, race_no, race_nm, procResult)
        
        # open_idが変わったらメール送信
        if rsRaceId.nexteof() == True or \
           rsRaceId.fields(trn_race_id.open_id.key).nextvalue != open_id:

            # mailMessage
            mmsg.mail_from = gv.netkeiberConfig.netkeibaMailConfig.mail_from
            mmsg.mail_to = gv.netkeiberConfig.netkeibaMailConfig.mail_to
            mmsg.subject = nkm.getSubject('race data acquisition process has been completed. open_id={0}'.format(open_id), procResult.name)
            mmsg.bodyText = nkm.getBodyTextByRaceInfoGetter(msgLine)
            # mailsend
            nkm.sendMail(mmsg)
            # msgLine初期化
            msgLine = []
    
    # 終了ログ
    Logger.logging.info('<< Race data acquisition process has been completed. year={0}'.format(str(year)))

    # psgr_trid - close
    psgr_trid.closeSession()
    
def AddMailMessage(nkm:netkeiberMailer, msgLine:list, rsRaceId:recset[trn_race_id], race_no, race_nm, procResult:nd.getterResult):
    
    """
    メール本文追加
    
    Parameters
    ----------
    nkm : netkeiberMailer
        netkeiberMailerインスタンス
    msgLine : list
        メッセージリスト
    rsRaceId : recset
        trn_race_id - recset
    race_no : str
        レース番号
    race_nm : str
        レース名
    procResult : getterResult
        処理結果
    """
    
    # メッセージヘッダ部挿入
    if len(msgLine) == 0:
        msgLine.append(nkm.nkMessageTemplate.mailMessage.importDataInfoHeaderH)
        msgLine.append(nkm.nkMessageTemplate.mailMessage.importDataInfoHeaderL)
    
    # msgArgs追加
    msgArgs = []
    msgArgs.append(rsRaceId.fields(trn_race_id.race_id.key).value)
    msgArgs.append(rsRaceId.fields(trn_race_id.racecourse_id.key).value)
    msgArgs.append(race_no)
    msgArgs.append(race_nm)
    msgArgs.append(rsRaceId.fields(trn_race_id.scraping_count.key).value)
    msgArgs.append(rsRaceId.fields(trn_race_id.get_time.key).value)
    msgArgs.append(procResult.name)
    # msgLine追加
    msgLine.append(nkm.nkMessageTemplate.mailMessage.importDataInfoDetailD.format(msgArgs))



import sys
import time
import LibHanger.Library.uwGetter as Getter
import LibHanger.Library.uwMath as uwMath
import netkeiber.Library.netkeiberConfiger as hc
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Library.uwDeclare import uwDeclare as en
from LibHanger.Models.recset import recset
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Getter.get_RaceData import getter_RaceData
from netkeiber.Register.register_RaceData import register_RaceData
from netkeiber.Models.trn_race_id import trn_race_id

def getRaceData(filePath, race_id):
    
    """
    レースデータ取得メソッド

    Parameters
    ----------
    filePath : str
        呼び出し元ファイルのパス
    race_id : str
        レースID
    """
    
    # 共通設定
    hc.netkeiberConfiger(gv, filePath, 'config')

    # uwPostgreSQL
    psgr_register = uwPostgreSQL(gv.config)
    psgr_trn_race_id = uwPostgreSQL(gv.config)
    psgr_getter_RaceData = uwPostgreSQL(gv.config)

    # レースデータ取得クラスインスタンス
    getRaceData = getter_RaceData(psgr_getter_RaceData)

    # 処理時間計測 - 開始
    start = time.perf_counter()

    # レースデータ取得
    psgr_trn_race_id.openSession(True)
    rsRaceId = recset[trn_race_id](trn_race_id)
    rsRaceId.setDA(psgr_trn_race_id)
    rsRaceId.filter(trn_race_id.race_id == race_id)
    if rsRaceId.eof() == False:
        
        # データ取得:パラメーター設定
        getRaceData.openInfoRead = False # 開催情報は取得しない
        race_id = rsRaceId.fields(trn_race_id.race_id.key).value
        racecourse_id = rsRaceId.fields(trn_race_id.racecourse_id.key).value
        # データ取得
        getRaceData.getData(race_id=race_id, racecourse_id=racecourse_id)

        # レコードセット退避
        rsRaceResult = getRaceData.raceResult.rsRaceResult
        rsRacdInfo = getRaceData.raceResult.rsRaceInfo
        rsMstHorse = getRaceData.raceResult.rsMstHourse
        rsMstJockey = getRaceData.raceResult.rsMstJockey
        rsMstTrainer = getRaceData.raceResult.rsMstTrainer
        rsMstHowner = getRaceData.raceResult.rsMstHowner
        rsHorseResult = getRaceData.horseResult.rsHorseResult
        rsOpenInfo = getRaceData.openInfo.rsOpenInfo
        rsRefundInfo = getRaceData.refundInfo.rsRefundInfo
        rsRaceIdLog = getRaceData.raceIdLog.rsLogRaceId

        # setDA
        rsRaceIdLog.setDA(psgr_register)

        # エラー有無判定
        if getRaceData.hasError == True:
            # レースIDログ登録
            upResult = rsRaceIdLog.upsert()
            if upResult.result == en.resultRegister.success:
                print('◎Regist Error Log Success')
            elif upResult.result == en.resultRegister.failure:
                print('☓Regist Error Log Failure')
            sys.exit()

        # reg
        regRaceData = register_RaceData(psgr_register, race_id)
        regRaceData.appendRecsetList(rsRaceResult)
        regRaceData.appendRecsetList(rsRacdInfo)
        regRaceData.appendRecsetList(rsMstHorse)
        regRaceData.appendRecsetList(rsMstJockey)
        regRaceData.appendRecsetList(rsMstTrainer)
        regRaceData.appendRecsetList(rsMstHowner)
        regRaceData.appendRecsetList(rsHorseResult)
        regRaceData.appendRecsetList(rsOpenInfo)
        regRaceData.appendRecsetList(rsRefundInfo)

        # update
        result = regRaceData.execUpdate()
        if result == en.resultRegister.success:
            print('◎DbUpdate - Success')
        elif result == en.resultRegister.failure:
            print('☓DbUpdate - Failed')

        # 処理時間 - 取得
        procTime = time.perf_counter() - start

        # スクレイピング回数、データ取得時間セット
        rsRaceId.editRow()
        rsRaceId.fields(trn_race_id.scraping_count.key).value = getRaceData.scrapingCount
        rsRaceId.fields(trn_race_id.get_time.key).value = uwMath.round(procTime, uwMath.fraction.round)
        rsRaceId.fields(trn_race_id.get_status.key).value = nd.getStatus.acquired.value
        rsRaceId.fields(trn_race_id.updinfo.key).value = Getter.getNow(Getter.datetimeFormat.updinfo)
        
        # レースID情報登録
        riResult = rsRaceId.upsert()
        if riResult.result == en.resultRegister.success:
            print('◎Regist RaceId Success')
        elif riResult.result == en.resultRegister.failure:
            print('☓Regist Error RaceId Failure')
        
        # レースIDログ登録
        rsRaceIdLog = regRaceData.raceIdLog.rsLogRaceId
        rsRaceIdLog.setDA(psgr_register)
        upResult = rsRaceIdLog.upsert()
        if upResult.result == en.resultRegister.success:
            print('◎Regist Log Success')
        elif upResult.result == en.resultRegister.failure:
            print('☓Regist Error Log Failure')

    # Close
    psgr_trn_race_id.closeSession()


import LibHanger.Models.modelFields as fld
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.elements import Null

# Baseクラス生成
Base = declarative_base()

class mst_peds(Base):
	
	# テーブル名
	__tablename__ = 'mst_peds'
	
	# スキーマ
	__table_args__ = {'schema': 'nkaz'}
	
	# 列定義
	peds_id = fld.CharFields(10,primary_key=True,default='')
	peds_seq = fld.NumericFields(2,0,primary_key=True,default=0)
	gen1_horse_id = fld.CharFields(12,default='')
	gen2_horse_id = fld.CharFields(12,default='')
	gen3_horse_id = fld.CharFields(12,default='')
	gen4_horse_id = fld.CharFields(12,default='')
	gen5_horse_id = fld.CharFields(12,default='')
	updinfo = fld.CharFields(40,default='')



import re
import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from enum import Enum
from decimal import Decimal
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup, ResultSet, Tag
from sqlalchemy.sql.elements import Null
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberException import getterError
from netkeiber.Library.netkeiberException import arrivalOrderValueError
from netkeiber.Library.netkeiberException import gettingValueError
from netkeiber.Models.trn_race_info import trn_race_info
from netkeiber.Models.trn_race_result import trn_race_result
from netkeiber.Models.mst_horse import mst_horse
from netkeiber.Models.mst_jockey import mst_jockey
from netkeiber.Models.mst_trainer import mst_trainer
from netkeiber.Models.mst_howner import mst_howner
from netkeiber.Getter.Base.baseGetter import baseGetter

class getter_trn_race_result(baseGetter):
    
    """
    レース結果取得クラス
    (trn_race_result)
    """
    
    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()

        # レコードセット初期化
        self.init_recset()

    def init_recset(self):
        
        """
        レコードセット初期化
        """
        
        # レコードセット初期化
        self.rsRaceResult = recset[trn_race_result](trn_race_result)
        self.rsRaceInfo = recset[trn_race_info](trn_race_info)
        self.rsMstHourse = recset[mst_horse](mst_horse)
        self.rsMstJockey = recset[mst_jockey](mst_jockey)
        self.rsMstTrainer = recset[mst_trainer](mst_trainer)
        self.rsMstHowner = recset[mst_howner](mst_howner)
        
    @Logger.loggerDecorator("getData",['race_id'])
    def getData(self, *args, **kwargs):
        
        """
        レース結果取得
        
        Parameters
        ----------
        kwargs : dict
            @race_id
                取得対象レースID
        """
        
        # レース結果をDataFrameで取得
        try:
            kwargs['getter'] = self
            result = self.getRaceDataToDataFrame(**kwargs)
        except:
            raise getterError
        return result
    
    @Logger.loggerDecorator("getRaceDataToDataFrame")
    def getRaceDataToDataFrame(self, *args, **kwargs):

        """
        レース結果取得
        
        Parameters
        ----------
        kwargs : dict
            @race_id
                取得対象レースID
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl, gv.netkeiberConfig.netkeibaUrlSearchKeyword.race)
        # 検索url(レース結果)
        raceUrl = urlParse.urljoin(rootUrl, kwargs.get('race_id'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(raceUrl)
        
        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    def getArrivalOrder(self, orderString:str):
        aor = gv.netkeiberConfig.settingValueStruct.ArrivalOrder
        icf = gv.netkeiberConfig.settingValueStruct.InCommingFlg
        retInComFlg = icf.defaultInLine
        
        if orderString.isnumeric(): # 着順が数値の場合
            
            # 着順取得
            orderInt = int(orderString)

        else: # 着順が文字の場合
            
            # 着順、入線フラグ取得
            orderInt, retInComFlg = (aor.exclusion, icf.exclusion) if orderString == aor.exclusionJp else \
                                    (aor.disqualification, icf.disqualification) if orderString == aor.disqualificationJp else \
                                    (aor.stop_competition, icf.stop_competition) if orderString == aor.stop_competitionJp else \
                                    (aor.raceCancel, icf.cancel_race) if orderString == aor.raceCancelJp1 or orderString == aor.raceCancelJp2 else \
                                    (int(orderString.replace('({0})'.format(icf.accretionJp), '')), icf.accretion) if icf.accretionJp in orderString else \
                                    (-1, retInComFlg)
        
        # 着順が取得できない場合例外とする
        if orderInt == -1:
            raise arrivalOrderValueError
        
        # 戻り値を返す
        return orderInt, retInComFlg
    
    def getTimeSecond(self, timeString:str) -> Decimal:
        timeSecond = 0
        
        try:
            # 分⇒秒変換
            minute = int(timeString.split(':')[0]) * 60
            
            # 秒数取得
            second = timeString.split(':')[1]
            
            # タイムを秒数に変換
            timeSecond = Decimal(minute) + Decimal(second)
            
        except Exception as e:
            Logger.logging.error('Time Convert Error : Value=' + timeString)
            Logger.logging.error(str(e))
            
        return timeSecond
    
    def getCorner(self, cornerString:str):
        corners = cornerString.split('-') if cornerString != '' else []
        return corners
    
    def getLast3furlong(self, last3furlongString:list):
        last3furlong = 0
        
        try:
            # 上がり3ハロン
            if (len(last3furlongString) > 0):
                tmpLast3furlong:str = last3furlongString[0]
                last3furlong = Decimal(tmpLast3furlong) if self.isdigitEx(tmpLast3furlong) else 0
            
        except Exception as e:
            Logger.logging.error('Last3furlong Get Error : Value=' + last3furlongString)
            Logger.logging.error(str(e))
            
        return last3furlong

    def getWinOdds(self, winOddsString:str):
        winOdds = 0
        
        try:
            # 単勝オッズ
            winOdds = Decimal(winOddsString)
            
        except Exception as e:
            Logger.logging.error('win_odds Get Error : Value=' + winOddsString)
            Logger.logging.error(str(e))
            
        return winOdds
    
    def getPopular(self, popularString:list):
        popular = 0
        
        try:
            # 人気
            if (len(popularString) > 0):
                tmpPopular:str = popularString[0]
                popular = int(tmpPopular) if self.isdigitEx(tmpPopular) else 0
                        
        except Exception as e:
            Logger.logging.error('popular Get Error : Value=' + popularString)
            Logger.logging.error(str(e))
            
        return popular
        
    def getHorseWeight(self, horseWeightString:str):
        horseWeight = 0
        
        try:
            # 馬体重(体重部分取得)
            horseWeightTemp = re.sub("\(.+?\)","",horseWeightString)
            if horseWeightTemp.isdigit():
                horseWeight = int(horseWeightTemp)
                
        except Exception as e:
            Logger.logging.error('horse_weight Get Error : Value=' + horseWeight)
            Logger.logging.error(str(e))
            
        return horseWeight
    
    def getWeightDiff(self, weightDiffString:str):
        weightDiff = 0
        
        try:
            # 馬体重(増減部分取得)
            weightDiffTemp:str = re.findall("(?<=\().+?(?=\))", weightDiffString)[0]
            if weightDiffTemp.replace('+','').replace('-','').isdigit():
                weightDiff = int(weightDiffTemp)
                
        except Exception as e:
            Logger.logging.error('horse_weight(diff) Get Error : Value=' + weightDiffString)
            Logger.logging.error(str(e))
            
        return weightDiff
    
    def getPrizeMoney(self, prizeMoneyString:str):
        prizeMoney = 0
        
        try:
            # 賞金
            prizeMoney_str = prizeMoneyString.replace(',','')
            try:
                prizeMoney = Decimal(prizeMoney_str) if prizeMoney_str != '' else 0
            except ValueError as e:
                raise
            
        except Exception as e:
            Logger.logging.error('prize_money Get Error : Value=' + prizeMoneyString)
            Logger.logging.error(str(e))
            raise gettingValueError
            
        return prizeMoney
    
    def getGrade(self, race_nm:str):
        grd = gv.netkeiberConfig.settingValueStruct.Grade
        grade = grd.defaultGrade
        
        # レース名の中にG1,G2,G3の文字列が存在するか
        if 'G1' in race_nm:
            grade = grd.g1Grade
        elif 'G2' in race_nm:
            grade = grd.g2Grade
        elif 'G3' in race_nm:
            grade = grd.g3Grade
                    
        return grade
    
    def getRaceClass(self, classString:str):
        raceClass = ''
        horseSign = ''
        raceClassTmp = classString.split('\xa0\xa0')
        if len(raceClassTmp) > 0:
            if len(raceClassTmp) == 2:
                raceClass = raceClassTmp[0]
                horseSign = raceClassTmp[1]
            elif len(raceClassTmp) == 1:
                raceClass = raceClassTmp[0]

        return raceClass, horseSign

    def getRaceDate(self, raceDateString:str):
        raceDate = raceDateString.replace('年','/').replace('月','/').replace('日','')
        return raceDate
    
    def getOpenInfo(self, openInfoString:str):
        return openInfoString

    def getHorseSign(self, horseSignString:str):
        return horseSignString
    
    def getDirection(self, directionString:str):
        drc = gv.netkeiberConfig.settingValueStruct.Direction
        direction = ''
        
        if drc.Steeplechase in directionString:
            direction = drc.Steeplechase
        elif '左' in directionString:
            direction = drc.left
        elif '右' in directionString:
            direction = drc.right
        
        return direction
    
    def getDistance(self, distanceString:str):
        distance = 0
        
        distanceTmp = str(re.sub(r"\D", "", distanceString))
        if distanceTmp.isdigit():
            distance = int(distanceTmp)
            
        return distance
    
    def getGroundKbn(self, groundKbnString:str):
        gkb = gv.netkeiberConfig.settingValueStruct.GroundKbn
        ground_kbn = gkb.Turf
        
        if gkb.SteeplechaseAbbr in groundKbnString and not (gkb.DirtAbbr in groundKbnString):
            ground_kbn = gkb.Steeplechase_Turf
        elif gkb.SteeplechaseAbbr in groundKbnString and not (gkb.TurfAbbr in groundKbnString):
            ground_kbn = gkb.Steeplechase_Dirt
        elif gkb.SteeplechaseAbbr in groundKbnString and gkb.TurfAbbr in groundKbnString and gkb.DirtAbbr in groundKbnString:
            ground_kbn = gkb.Steeplechase_TurfAndDirt
        elif gkb.TurfAbbr in groundKbnString:
            ground_kbn = gkb.Turf
        elif gkb.DirtAbbr in groundKbnString:
            ground_kbn = gkb.Dirt
        return ground_kbn
    
    def getWeather(self, weatherString:str):
        weather = ''
    
        weatherTmp = weatherString.split(' : ')
        if len(weatherTmp) > 0:
            weather = weatherTmp[1]
        return weather
    
    def getGroundCond(self, groundCondString:str):
        groundCond = ''
        groundCond_d = ''
        
        groundCondTmp = groundCondString.split('\xa0\xa0')
        if len(groundCondTmp) > 0:
            if len(groundCondTmp) == 1:
                groundCond = groundCondTmp[0].split(' : ')[1]
            elif len(groundCondTmp) == 2:
                groundCond = groundCondTmp[0].split(' : ')[1]
                groundCond_d = groundCondTmp[1].split(' : ')[1]

        return groundCond, groundCond_d
    
    def getRaceTime(self, raceTimeString:str):
        raceTime = ''
        
        raceTimeTmp = raceTimeString.split(' : ')
        if len(raceTimeTmp) > 0:
            raceTime = raceTimeTmp[1]
        return raceTime
    
    def getRefundId(self, raceNum):
        refundId = ''

        if raceNum:
            refundInfo = raceNum.find_all('p')[1].find_all('a')[0]
            refundId = str(refundInfo.get('href')).split('/')[4]
        
        return refundId
    
    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """
        
        class raceResultCol(Enum):
            
            """
            レース結果列インデックス
            """
            
            arrival_order = 0
            """ 着順 """

            frameNo = 1
            """ 枠番 """
            
            horseNo = 2
            """ 馬番 """
            
            horseId = 3
            """ 競走馬ID """

            sexAge = 4
            """ 性齢 """

            weight = 5
            """ 斤量 """

            jockey_id = 6
            """ 騎手ID """
            
            time = 7
            """ タイム """

            arr_diff = 8
            """ 着差 """

            corners = 10
            """ 通過順 """
            
            last3furlong = 11
            """ あがり3ハロン"""

            win_odds = 12
            """ 単勝オッズ """
            
            popular = 13
            """ 人気 """

            horse_weight = 14
            """ 馬体重 """
            
            trainer_id = 18
            """ 調教師ID """
            
            howner_id = 19
            """ 馬主ID """

            prize_money = 20
            """ 賞金 """

        def __init__(self, _config: netkeiberConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : netkeiberConfig
                    共通設定
            """
            
            super().__init__(_config)
            
            self.config = _config
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup

        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            レース結果をDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @race_id
                    取得対象レースID
            """

            # レースID取得
            race_id:str = kwargs.get('race_id')
            
            # getterインスタンス取得
            self.bc:getter_trn_race_result = kwargs.get('getter')
            
            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]

            # レース結果class取得
            raceTables = soup.find(class_="race_table_01").find_all('tr')
            
            # レース情報class取得
            dataIntro = soup.find(class_="data_intro")
            
            # 払戻情報class取得
            raceNum = soup.find(class_="race_num")
    
            if dataIntro:
                
                # raceData取得
                raceData = dataIntro.find(class_="racedata")
                if raceData:
    
                    # レース番号
                    race_no = raceData.find('dt').text.replace('\n','').replace('R','').rstrip()
                    
                    # レース名
                    race_nm = raceData.find('h1').text
                    
                    # グレード
                    grade = self.bc.getGrade(race_nm)
                    
                # diary_snap_cut取得
                diarySnapCut = dataIntro.find_all('diary_snap_cut')
                if diarySnapCut:
                    
                    # コース概要
                    course_summarys = str(diarySnapCut[0].find('span').text)
                    
                    diary = course_summarys.split('\xa0/\xa0')
                    if len(diary) > 0:
                        
                        # コース概要
                        course_summary = diary[0]
                        # 向き 
                        direction = self.bc.getDirection(diary[0])
                        # 距離
                        distance = self.bc.getDistance(diary[0])
                        # 芝/ダート
                        ground_kbn = self.bc.getGroundKbn(diary[0])
                        # 天候
                        weather = self.bc.getWeather(diary[1])
                        # 馬場状態
                        ground_cond, ground_cond_d = self.bc.getGroundCond(diary[2])
                        # 発送時刻
                        race_time = self.bc.getRaceTime(diary[3])

                # 払戻ID
                refund_id = self.bc.getRefundId(raceNum)

                # レース概要取得
                # 取得例)2021年12月12日 6回阪神4日目 2歳オープン&nbsp;&nbsp;(国際) 牝(指)(馬齢)
                # [0]:日付
                # [1]:開催情報
                # [2]:クラス
                # [3]:馬名記号
                raceInfoSummaryString = str(dataIntro.find_all('p')[1].text)
                raceInfoSummary = raceInfoSummaryString.split(' ')
                
                # 日付
                race_date = self.bc.getRaceDate(raceInfoSummary[0])

                # 開催情報
                open_info = self.bc.getOpenInfo(raceInfoSummary[1])

                # クラス名、馬名記号
                class_nm, horse_sign = self.bc.getRaceClass(raceInfoSummary[2])

                # レース情報model用意
                raceInfo = recset[trn_race_info](trn_race_info)
                
                # Modelに追加
                raceInfo.newRow()
                raceInfo.fields(trn_race_info.race_id.key).value = race_id
                raceInfo.fields(trn_race_info.race_no.key).value = race_no
                raceInfo.fields(trn_race_info.race_nm.key).value = race_nm
                raceInfo.fields(trn_race_info.grade.key).value = grade
                raceInfo.fields(trn_race_info.course_summary.key).value = course_summary
                raceInfo.fields(trn_race_info.direction.key).value = direction
                raceInfo.fields(trn_race_info.distance.key).value = distance
                raceInfo.fields(trn_race_info.ground_kbn.key).value = ground_kbn
                raceInfo.fields(trn_race_info.weather.key).value = weather
                raceInfo.fields(trn_race_info.ground_cond.key).value = ground_cond
                raceInfo.fields(trn_race_info.ground_cond_d.key).value = ground_cond_d
                raceInfo.fields(trn_race_info.race_date.key).value = race_date
                raceInfo.fields(trn_race_info.race_time.key).value = race_time
                raceInfo.fields(trn_race_info.open_info.key).value = open_info
                raceInfo.fields(trn_race_info.class_nm.key).value = class_nm
                raceInfo.fields(trn_race_info.horse_sign.key).value = horse_sign
                raceInfo.fields(trn_race_info.head_count.key).value = len(raceTables) - 1
                raceInfo.fields(trn_race_info.racecourse_id.key).value = race_id[4:6]
                raceInfo.fields(trn_race_info.refund_id.key).value = refund_id
                raceInfo.fields(trn_race_info.updinfo.key).value = self.bc.getUpdInfo()
                
            if raceTables:
                
                # 先着馬のタイム
                bef_time_second = 0
                # 1着馬との着差
                tarr_diff_num = 0

                # レース結果model用意 
                raceResult = recset[trn_race_result](trn_race_result)

                # 競走馬マスタmodel用意
                mstHorse = recset[mst_horse](mst_horse)

                # 騎手マスタmodel用意
                mstJokey = recset[mst_jockey](mst_jockey)

                # 調教師マスタmodel用意
                mstTrainer = recset[mst_trainer](mst_trainer)

                # 馬主マスタmodel用意
                mstHowner = recset[mst_howner](mst_howner)
                
                for index in range(len(raceTables)):
                    if index == 0 : continue
                    try:
                        dataTbl:Tag = raceTables[index]
                        row:ResultSet = dataTbl.find_all('td')
                        drow = self.getRow(row)
                        # 着順
                        arrival_order, incommingFlg = self.bc.getArrivalOrder(drow[self.raceResultCol.arrival_order.value].text)
                        # 枠番
                        frameNo = Decimal(drow[self.raceResultCol.frameNo.value].text)
                        # 馬番
                        horseNo = Decimal(drow[self.raceResultCol.horseNo.value].text)
                        # 馬ID
                        horseId = str(drow[self.raceResultCol.horseId.value].find_all('a')[0].get('href')).split('/')[2]
                        # 馬名
                        horseNm = drow[self.raceResultCol.horseId.value].text
                        # 性齢
                        sexAge = drow[self.raceResultCol.sexAge.value].text
                        # 斤量
                        weight = Decimal(drow[self.raceResultCol.weight.value].text)
                        # 騎手ID
                        jockey_id = str(drow[self.raceResultCol.jockey_id.value].find_all('a')[0].get('href')).split('/')[4]
                        # 騎手名
                        jockey_nm = drow[self.raceResultCol.jockey_id.value].text
                        # タイム
                        time = drow[self.raceResultCol.time.value].text
                        # タイム(数値)
                        time_second = self.bc.getTimeSecond(time)
                        # 着差
                        arr_diff = drow[self.raceResultCol.arr_diff.value].text
                        # 着差(秒)
                        barr_diff_num = time_second - bef_time_second if arr_diff != '' else 0
                        # 着差(累計)
                        tarr_diff_num += barr_diff_num
                        # 通過順
                        corners = self.bc.getCorner(drow[self.raceResultCol.corners.value].text)
                        # 上がり3ハロン
                        last3furlong = self.bc.getLast3furlong(drow[self.raceResultCol.last3furlong.value].find_all('span')[0].contents)
                        # 単勝オッズ
                        win_odds = self.bc.getWinOdds(drow[self.raceResultCol.win_odds.value].text)
                        # 人気
                        popular = self.bc.getPopular(drow[self.raceResultCol.popular.value].find_all('span')[0].contents)
                        # 馬体重
                        horse_weight = self.bc.getHorseWeight(drow[self.raceResultCol.horse_weight.value].text)
                        # 馬体重増減
                        weight_diff = self.bc.getWeightDiff(drow[self.raceResultCol.horse_weight.value].text)
                        # 調教師ID
                        trainer_id = str(drow[self.raceResultCol.trainer_id.value].find_all('a')[0].get('href')).split('/')[4]
                        # 調教師名
                        trainer_nm = drow[self.raceResultCol.trainer_id.value].text
                        trainer_nm = trainer_nm.replace('\n','')
                        # 馬主ID
                        howner_id = str(drow[self.raceResultCol.howner_id.value].find_all('a')[0].get('href')).split('/')[4]
                        # 馬主名
                        howner_nm = drow[self.raceResultCol.howner_id.value].text
                        howner_nm = howner_nm.replace('\n','')
                        # 賞金
                        prize_money = self.bc.getPrizeMoney(drow[self.raceResultCol.prize_money.value].text)
                        
                        # Modelに追加
                        raceResult.newRow()
                        raceResult.fields(trn_race_result.race_id.key).value = race_id
                        raceResult.fields(trn_race_result.horse_no.key).value = horseNo
                        raceResult.fields(trn_race_result.arrival_order.key).value = arrival_order
                        raceResult.fields(trn_race_result.incoming_flg.key).value = incommingFlg
                        raceResult.fields(trn_race_result.horse_id.key).value = horseId
                        raceResult.fields(trn_race_result.horse_nm_en.key).value = horseNm if horseId == '' else ''
                        raceResult.fields(trn_race_result.frame_no.key).value = frameNo
                        raceResult.fields(trn_race_result.sex_age.key).value = sexAge
                        raceResult.fields(trn_race_result.weight.key).value = weight
                        raceResult.fields(trn_race_result.jockey_id.key).value = jockey_id
                        raceResult.fields(trn_race_result.time.key).value = time
                        raceResult.fields(trn_race_result.time_second.key).value = time_second
                        raceResult.fields(trn_race_result.arr_diff.key).value = arr_diff
                        raceResult.fields(trn_race_result.barr_diff_num.key).value = barr_diff_num
                        raceResult.fields(trn_race_result.tarr_diff_num.key).value = tarr_diff_num
                        raceResult.fields(trn_race_result.corner1.key).value = corners[0] if len(corners) >= 1 else Null()
                        raceResult.fields(trn_race_result.corner2.key).value = corners[1] if len(corners) >= 2 else Null()
                        raceResult.fields(trn_race_result.corner3.key).value = corners[2] if len(corners) >= 3 else Null()
                        raceResult.fields(trn_race_result.corner4.key).value = corners[3] if len(corners) >= 4 else Null()
                        raceResult.fields(trn_race_result.last3furlong.key).value = last3furlong
                        raceResult.fields(trn_race_result.win_odds.key).value = win_odds
                        raceResult.fields(trn_race_result.popular.key).value = popular
                        raceResult.fields(trn_race_result.horse_weight.key).value = horse_weight
                        raceResult.fields(trn_race_result.weight_diff.key).value = weight_diff
                        raceResult.fields(trn_race_result.trainer_id.key).value = trainer_id
                        raceResult.fields(trn_race_result.howner_id.key).value = howner_id
                        raceResult.fields(trn_race_result.prize_money.key).value = prize_money
                        raceResult.fields(trn_race_result.updinfo.key).value = self.bc.getUpdInfo()

                        # 競争馬マスタ追加
                        mstHorse.newRow()
                        mstHorse.fields(mst_horse.horse_id.key).value = horseId
                        mstHorse.fields(mst_horse.horse_nm.key).value = horseNm
                        mstHorse.fields(mst_horse.birthday.key).value = Null()
                        mstHorse.fields(mst_horse.updinfo.key).value = self.bc.getUpdInfo()

                        # 騎手マスタ追加
                        mstJokey.newRow()
                        mstJokey.fields(mst_jockey.jockey_id.key).value = jockey_id
                        mstJokey.fields(mst_jockey.jockey_nm.key).value = jockey_nm
                        mstJokey.fields(mst_jockey.birthday.key).value = Null()
                        mstJokey.fields(mst_jockey.updinfo.key).value = self.bc.getUpdInfo()

                        # 調教師マスタ追加
                        mstTrainer.newRow()
                        mstTrainer.fields(mst_trainer.trainer_id.key).value = trainer_id
                        mstTrainer.fields(mst_trainer.trainer_nm.key).value = trainer_nm
                        mstTrainer.fields(mst_trainer.birthday.key).value = Null()
                        mstTrainer.fields(mst_trainer.updinfo.key).value = self.bc.getUpdInfo()

                        # 馬主マスタ追加
                        mstHowner.newRow()
                        mstHowner.fields(mst_howner.howner_id.key).value = howner_id
                        mstHowner.fields(mst_howner.howner_nm.key).value = howner_nm
                        mstHowner.fields(mst_howner.updinfo.key).value = self.bc.getUpdInfo()
                        
                        # コンソール出力
                        print('着順={0}'.format(str(arrival_order)))
                        print('枠番={0}'.format(str(frameNo)))
                        print('馬番={0}'.format(str(horseNo)))
                        print('馬ID={0}'.format(horseId))
                        print('馬名={0}'.format(horseNm))

                        # 当該馬のタイム退避
                        bef_time_second = time_second
                        
                    except arrivalOrderValueError as aoException: # 着順例外
                        Logger.logging.error(str(aoException))
                        raise 
                    except gettingValueError as gvException: # 値例外
                        Logger.logging.error(str(gvException))
                        raise 
                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                        raise 
                
                # レコードセットマージ
                self.bc.rsRaceResult.merge(raceResult)
                self.bc.rsRaceInfo.merge(raceInfo)
                self.bc.rsMstHourse.merge(mstHorse)
                self.bc.rsMstJockey.merge(mstJokey)
                self.bc.rsMstTrainer.merge(mstTrainer)
                self.bc.rsMstHowner.merge(mstHowner)
                
                # 戻り値を返す
                return raceResult.getDataFrame()
                


import os
import LibHanger.Library.uwLogger as Logger
from netkeiber.Register.Base.baseRegister import baseRegister
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Models.recset import recset

class register_mst_racecourse(baseRegister):
    
    """
    競馬場マスタ登録クラス
    """
    
    def __init__(self, __psgr: uwPostgreSQL, __recset: recset):

        """
        コンストラクタ
        """

        # 基底側コンストラクタ
        super().__init__(__psgr, __recset)
    
    def createInitialData(self, initDataSQLFilePath):
        
        """
        競馬場マスタ初期データ作成
        
        Parameters
        ----------
        initDataSQLFilePath : str
            競馬場マスタ初期データ作成SQLファイルパス
        """
        
        # ファイルパスが見つからない場合処理を中断
        if not os.path.exists(initDataSQLFilePath) : return
        
        # 実行するSQLファイル読込
        with open(initDataSQLFilePath) as sqlfile:
            
            # ファイル読込
            sqldata = sqlfile.read()
            # SQL実行
            result = self.execSqlfile(self.__psgr, sqldata)
            # 結果判定
            if result == self.resultRegister.success:
                Logger.logging.info('execute ok file={}'.format(initDataSQLFilePath))
            elif result == self.resultRegister.failure:
                Logger.logging.error('execute error file={}'.format(initDataSQLFilePath))



import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from pandas.core.frame import DataFrame
from bs4 import BeautifulSoup
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberGlobals import *
from netkeiber.Library.netkeiberException import racdIdCheckError
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Models.trn_race_id import trn_race_id
from netkeiber.Getter.Base.baseGetter import baseGetter

class getter_trn_race_id(baseGetter):
    
    """
    レースID情報取得クラス
    (trn_race_id)
    """

    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()

        # レコードセット初期化
        self.init_recset()
        
    def init_recset(self):
        
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsRaceId = recset[trn_race_id](trn_race_id)

    @Logger.loggerDecorator("getData",['year'])
    def getData(self, *args, **kwargs):
        
        """
        開催情報取得
        
        Parameters
        ----------
        kwargs : dict
            @year
                開催年
            @month
                開催月
        """
        
        # 開催日情報をDataFrameで取得
        kwargs['getter'] = self
        dfRaceIdCal = self.getRaceIdCalToDataFrame(**kwargs)

        # 取得した開催日をループして1つずつRaceIdを取り出す
        for index, item in dfRaceIdCal.iterrows():
            kwargs['open_id'] = item[trn_race_id.open_id.name]
            kwargs['racecourse_id'] = item[trn_race_id.racecourse_id.name]
            self.getRaceIdToDataFrame(**kwargs)
    
    @Logger.loggerDecorator("getRaceIdDataToDataFrame")
    def getRaceIdCalToDataFrame(self, *args, **kwargs):

        """
        レースID情報取得(カレンダー取得)
        
        Parameters
        ----------
        kwargs : dict
            @year
                開催年度
            @month
                開催月
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl_race, gv.netkeiberConfig.netkeibaUrlSearchKeyword.race_id_cal)
        # 検索url(レースID情報[カレンダー])
        raceIdCalUrl = rootUrl.format(kwargs.get('year'), kwargs.get('month'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(raceIdCalUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    @Logger.loggerDecorator("getRaceIdToDataFrame")
    def getRaceIdToDataFrame(self, *args, **kwargs):

        """
        レースID情報取得
        
        Parameters
        ----------
        kwargs : dict
            @racecourse_id
                競馬場ID
            @open_id
                開催ID
        """
        
        # 検索url(ルート)
        rootUrl = urlParse.urljoin(gv.netkeiberConfig.netkeibaUrl, gv.netkeiberConfig.netkeibaUrlSearchKeyword.open)
        # 検索url(開催情報)
        raceIdUrl = urlParse.urljoin(rootUrl, kwargs.get('racecourse_id') + '/' + kwargs.get('open_id'))

        # スクレイピング準備
        self.wdc.settingScrape()

        # ページロード
        self.wdc.browserCtl.loadPage(raceIdUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)
        
    class beautifulSoup(browserContainer.beautifulSoup):
        
        """
        ブラウザコンテナ:beautifulSoup
        """

        def __init__(self, _config: netkeiberConfig):
            
            """
            コンストラクタ
            
            Parameters
            ----------
                _config : netkeiberConfig
                    共通設定
            """

            super().__init__(_config)
            
            self.config = _config
            self.cbCreateSearchResultDataFrameByBeutifulSoup = self.createSearchResultDataFrameByBeutifulSoup
        
        def getOpenCal(self, soup:BeautifulSoup, *args, **kwargs):
            
            """
            開催日カレンダー情報を取得する
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @year
                    開催年度
            """

            # getterインスタンス取得
            self.bc:getter_trn_race_id = kwargs.get('getter')

            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            tables = soup.find(class_="Calendar_Table").find_all(class_='RaceCellBox')

            if tables:
                
                # レースIDカレンダー情報model用意
                raceIdCalInfo = recset[trn_race_id](trn_race_id)
                
                for index in range(len(tables)):
                    try:
                        # 開催ID
                        open_id_a = tables[index].find_all('a')
                        if open_id_a:
                            open_id_href = open_id_a[0].get('href')
                            open_id = str(open_id_href).split('=')[1]
                        else:
                            continue
                        
                        jyo_name = tables[index].find_all(class_="JyoName")
                        for index in range(len(jyo_name)):
                            
                            # 競馬場名
                            course_nm = jyo_name[index].text
                            # 競馬場ID
                            racecourse_id = gv.netkeiberConfig.courseList[course_nm]
                            
                            # Modelに追加
                            raceIdCalInfo.newRow()
                            raceIdCalInfo.fields(trn_race_id.race_id.key).value = '*'
                            raceIdCalInfo.fields(trn_race_id.racecourse_id.key).value = racecourse_id
                            raceIdCalInfo.fields(trn_race_id.open_id.key).value = open_id
                            raceIdCalInfo.fields(trn_race_id.updinfo.key).value = self.bc.getUpdInfo()

                            # コンソール出力
                            print('競馬場ID={0}'.format(racecourse_id))
                            print('開催ID={0}'.format(open_id))
                        
                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                
                return raceIdCalInfo.getDataFrame()

        def getRaceId(self, soup:BeautifulSoup, *args, **kwargs):
            
            """
            レースID情報を取得する
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @open_id
                    開催ID
                @racecourse_id
                    競馬場ID
            """

            # getterインスタンス取得
            self.bc:getter_trn_race_id = kwargs.get('getter')
            
            # 開催ID取得
            open_id:str = kwargs.get('open_id')

            # 競馬場ID取得
            racecourse_id:str = kwargs.get('racecourse_id')
            
            # スクレイピング結果から改行ｺｰﾄﾞを除去
            [tag.extract() for tag in soup(string='\n')]
            
            # class取得
            tables = soup.find(class_="race_table_01").find_all('tr')

            if tables:
                
                # レースID情報model用意
                raceIdInfo = recset[trn_race_id](trn_race_id)
                
                for index in range(len(tables)):
                    if index == 0 : continue
                    try:
                        # tdタグ取得
                        row = tables[index].find_all('td')

                        # レースID
                        race_id = str(row[1].find_all('a')[0].get('href')).split('/')[2]

                        # レースIDが数値で構成されていなければ例外を発生させる
                        if not race_id.isdigit():
                            raise racdIdCheckError
                        
                        # Modelに追加
                        raceIdInfo.newRow()
                        raceIdInfo.fields(trn_race_id.race_id.key).value = race_id
                        raceIdInfo.fields(trn_race_id.racecourse_id.key).value = racecourse_id
                        raceIdInfo.fields(trn_race_id.open_id.key).value = open_id
                        raceIdInfo.fields(trn_race_id.scraping_count.key).value = 0
                        raceIdInfo.fields(trn_race_id.get_time.key).value = 0
                        raceIdInfo.fields(trn_race_id.get_status.key).value = nd.getStatus.unacquired.value
                        raceIdInfo.fields(trn_race_id.updinfo.key).value = self.bc.getUpdInfo()
                        
                        # コンソール出力
                        print('レースID={0}'.format(race_id))
                        print('競馬場ID={0}'.format(racecourse_id))
                        print('開催日={0}'.format(open_id))
                    
                    except racdIdCheckError as e:
                        Logger.logging.error(str(e))
                        Logger.logging.error('race_id Value={0}'.format(race_id))
                        Logger.logging.error('open_id Value={0}'.format(open_id))

                    except Exception as e: # その他例外
                        Logger.logging.error(str(e))
                
                # レコードセットマージ
                self.bc.rsRaceId.merge(raceIdInfo)
                
                # 戻り値をDataFrameで返却
                return raceIdInfo.getDataFrame()
                    
        def createSearchResultDataFrameByBeutifulSoup(self, soup:BeautifulSoup, *args, **kwargs) -> DataFrame:
            
            """
            レースID情報をDataFrameで返す(By BeutifulSoup)
            
            Parameters
            ----------
            soup : BeautifulSoup
                BeautifulSoupオブジェクト
            
            kwargs : dict
                @year
                    開催年度
                @month
                    開催月
                @kaisai_date
                    開催日
            """

            if kwargs.get('open_id') == None:
                return self.getOpenCal(soup, *args, **kwargs)
            else:
                return self.getRaceId(soup, *args, **kwargs)



from LibHanger.Models.recset import recset
from netkeiber.Library.netkeiberController import netkeibaBrowserController

class baseGetter(netkeibaBrowserController):
    
    """
    Getter基底
    """
    
    __dictResult = {}
    """ 処理結果 """

    def __init__(self) -> None:
        
        """
        コンストラクタ
        """
        
        super().__init__()

        self.__dictResult = {}
        
    @property
    def scrapingCount(self):
        
        """
        スクレイピング回数
        """
        
        return self.wdc.browserCtl.loadPageCount if not self.wdc.browserCtl is None else 0

    class getterResult():
        
        """
        Getter処理結果
        """
        
        def __init__(self, __recordCount:int, __recSet:recset):
        
            """
            コンストラクタ
            """
            
            self.recordCount:int = __recordCount
            """ 取得件数 """
            
            self.recSet:recset = __recSet
            """ 取得したレコードセット """
    
    def addResult(self, __key, __recordCount:int, __recSet:recset):
        
        """
        処理結果追加
        """
        
        self.__dictResult[__key] = self.getterResult(__recordCount, __recSet)
        
        return self.__dictResult



from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from netkeiber.Register.Base.baseRegister import baseRegister

class register_RaceData(baseRegister):
    
    """
    レースデータ一括登録クラス
    """
    
    def __init__(self, __psgr:uwPostgreSQL, __race_id = '*'):
        
        """
        コンストラクタ
        """
        
        # 基底側コンストラクタ
        super().__init__(__psgr, __race_id)
    